<?php

require_once("Product.php");

class Config
{
	private $filename_default = "config.xml";
	private $filename;
	private $data; // raw simplexml data

	function __construct($file = null)
	{
		global $options;

		if (substr($file, -1) == '/')
			throw new Mirror_Config_Exception("Wrong config file name: $file");

		$file ? $file : ($file = $options['tmpdir'] . "/" . $this->filename_default);
		$this->filename = $file;

		// Parse config file
		$this->pull();

		return $this;
	}

	private function pull()
	{
		echo_try("read the config: " . $this->filename);
		if (!file_exists($this->filename)) {
			$xml = '<?xml version="1.0" encoding="UTF-8" ?><xml/>';
			$this->data = simplexml_load_string($xml);
			echo_ok();
			return;
		}

		$this->data = simplexml_load_file($this->filename);
		if (!$this->data) {
			throw new Mirror_Exception("Unable to load the config file '"
				. $this->filename . "'");
		}
		echo_ok();
	}

	private function push()
	{
		save_config($this->filename, $this->data);
	}

	private function read_release_version($release)
	{
		global $options;

		$product = Product::factory(Types::PLESK);
		$data = $product->get_data();
		$version = $data[$release]['info']['version'];
		return $version;
	}

	public function add($release, $os)
	{
		global $options;

		if (!isset($release) || !isset($os) || empty($release) || empty($os))
			throw new Mirror_Cmd_Exception("Some parameters are not defined");

		$elem = '';
		if (!$this->data->release) {
			$elem = $this->data->addChild('release');
			$elem->addAttribute('name', $release);
			$elem->addAttribute('version', $this->read_release_version($release));
		} else {
			$flag = false;
			for ($i = 0; $this->data->release[$i]; $i++) {
				$elem = $this->data->release[$i];
				if ($elem->attributes()->name == $release) {
					$flag = true;
					break;
				}
			}
			if (!$flag) {
				$elem = $this->data->addChild('release');
				$elem->addAttribute('name', $release);
				$elem->addAttribute('version', $this->read_release_version($release));
			}
		}

		if ($this->os_exists($elem, $os)) {
			echo_try("add OS '$os' into config file");
			echo_skip("already exists");
			return;
		}

		if ($os == "all") {
			$product = Product::factory(Types::PLESK);
			$oslist = $product->get_os_list($release);
			echo_try("add all OSes for release '$release' into config file");
			foreach ($oslist AS $key) {
				if (!$this->os_exists($elem, $key)) {
					$child = $elem->addChild('os');
					$child->addAttribute('name', $key);
				}
			}
			echo_ok();
			$this->push();
			return;
		}

		echo_try("add OS '$os' into config file");
		$child = $elem->addChild('os');
		$child->addAttribute('name', $os);
		echo_ok();

		$this->push();
	}

	private function os_exists($elem, $os)
	{
		if (!$elem->os)
			return false;

		foreach ($elem->os AS $key) {
			// Element already exists
			if ($os == $key->attributes()->name)
				return true;
		}
		return false;
	}

	public function remove($release, $os)
	{
		echo_try("remove OS '$os' from config file");
		if (!isset($release) || !isset($os) || empty($release) || empty($os))
			throw new Mirror_Cmd_Exception("Some parameters are not defined");

		// Already deleted
		if (!$this->data->release) {
			echo_skip("not exists");
			return;
		}

		foreach ($this->data->release AS $elem) {
			if ($elem->attributes()->name == $release) {
				if ($os == "all") {
					$domRef = dom_import_simplexml($elem);
					$domRef->parentNode->removeChild($domRef);
					break;
				}

				for ($j = 0; $elem->os[$j]; $j++) {
					if ($elem->os[$j]->attributes()->name == "$os")
						unset($elem->os[$j]);
				}

				// Remove empty release section
				if (!$elem->os) {
					$domRef = dom_import_simplexml($elem);
					$domRef->parentNode->removeChild($domRef);
					break;
				}
			}
		}
		echo_ok();
		$this->push();
	}

	public function get_release_version($release)
	{
		if (!$this->data->release)
			return;

		foreach ($this->data->release AS $rel) {
			if ($rel->attributes()->name == $release)
				return (string)$rel->attributes()->version;
		}

		return;
	}

	public function get_release_list()
	{
		$list = array();

		foreach ($this->data->release AS $release)
			$list[] = (string)$release->attributes()->name;

		return $list;
	}

	public function get_os_list($release)
	{
		$list = array();
		$rels = $this->get_release_list();
		$flag = false;

		foreach ($rels AS $rel) {
			if ($rel == $release) {
				$flag = true;
				break;
			}
		}
		if (!$flag)
                        throw new Mirror_Config_Exception("Unknown release '$release'");

		$data = objectsIntoArray($this->data);
		if (isset($data['release']) && !isset($data['release'][0])) {
			$elem = $data['release']['os'];
			if (isset($elem) && !isset($elem[0])) {
				$list[] = $elem['@attributes']['name'];
				return $list;
			}

			for ($i = 0; $elem[$i]; $i++)
				$list[] = $elem[$i]['@attributes']['name'];

			return $list;
		}

		for($i = 0; $data['release'][$i]; $i++) {
			$elem = $data['release'][$i];
			if ($release == $elem['@attributes']['name']) {
				if (isset($elem['os']) && !isset($elem['os'][0])) {
					$list[] = $elem['os']['@attributes']['name'];
					break;
				}
				for ($j = 0; $elem['os'][$j]; $j++)
					$list[] = $elem['os'][$j]['@attributes']['name'];
				break;
			}
		}
		return $list;
	}

	public function check()
	{
		if (!file_exists($this->filename))
			throw new Mirror_Config_Exception("Config file: " . $this->filename . " doesn't found");

		$product = Product::factory(Types::PLESK);
		$releases = $product->get_release_list();
		$my_releases = $this->get_release_list();
		$opts = getCmdOptions();

		foreach($my_releases AS $my_release) {
			$flag = false;
			foreach($releases AS $release) {
				$winflag = preg_match("/^(.*)_WIN$/", $release);
				if (($winflag && !$opts->is_win()) || (!$winflag && $opts->is_win()))
					throw new Mirror_Config_Exception("Mixed configuration (Windows/Unix) has been used");

				if ($my_release == $release) {
					$flag = true;
					break;
				}
			}
			if (!$flag)
				throw new Mirror_Config_Exception("Unknown release '" . $my_release . "' has been defined in config.");

			$oses = $product->get_os_list($my_release);
			$my_oses = $this->get_os_list($my_release);

			foreach ($my_oses AS $my_os) {
				if ($my_os == "all") {
					$flag = true;
					break;
				}
				$flag = false;
				foreach ($oses AS $os) {
					if ($os == $my_os) {
						$flag = true;
						break;
					}
				}
				if (!$flag)
					throw new Mirror_Config_Exception("Unknown OS '" . $my_os . "' has been defined in config for release " . $my_release);
			}
		}
	}

	public function export()
	{
		$opts = getCmdOptions();
		$action = Action::factory();
		$action->upload($this->filename, $opts->get_path() . "/mirror.xml" ,0644);
	}

	public function import()
	{
		$opts = getCmdOptions();
		$action = Action::factory();
		$action->download($opts->get_path() . "/mirror.xml", $this->filename ,0644);
	}

}
